<?php
namespace OrganizationsManagement\Controller;
use Zeedhi\Framework\Controller\Simple;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Request;
use Zeedhi\Framework\DTO\Response;
use Zeedhi\Framework\DataSource;
use OrganizationsManagement\Service;
use OrganizationsManagement\Util\Messages;
use OrganizationsManagement\Util\Utilities;
use OrganizationsManagement\Util\InstanceProvider;

class Api extends Simple
{
    /** @var Service\Api $api */
    protected $apiService;

    public function __construct(Service\Api $api){
        $this->apiService = $api;
    }
    
    public function saveOrganizationWithProductsAndElementsRules(DTO\Request\DataSet $request, DTO\Response $response){
        try {
            $row                     = $request->getDataSet()->getRows()[0];
            $products                = $row['products'];
            $clientId                = $row['client']['code'];
            $operatorName            = $row['client']['operatorName'];
            $operatorEmail           = $row['client']['operatorEmail'];
            $requesterUserId         = $row['requester']['userId'];
            $requesterOrganizationId = $row['requester']['organizationId'];
            $initialDate             = isset($row['initialDate'])      ? $row['initialDate']      : null;
            $expirationDate          = isset($row['expirationDate'])   ? $row['expirationDate']   : null;
            $isExtendContract        = isset($row['isExtendContract']) ? $row['isExtendContract'] : null;
            $contractCode            = isset($row['contractCode'])     ? $row['contractCode']     : null;
            $oldContractCode         = isset($row['oldContractCode'])  ? $row['oldContractCode']  : null;
            $oldExpirationDate         = isset($row['oldExpirationDate'])  ? $row['oldExpirationDate']  : null;
            $oldInitialDate          = isset($row['oldInitialDate'])  ? $row['oldInitialDate']  : null;
            $result = $this->apiService->saveOrganizationWithProductsAndElementsRules($requesterOrganizationId, $requesterUserId,
                $clientId, $operatorName, $operatorEmail, $products, $expirationDate, $isExtendContract, $contractCode, $initialDate, $oldContractCode, $oldInitialDate, $oldExpirationDate);
            $response->addDataSet(new DataSource\DataSet('clientData', $result));
        } catch(\Exception $ex) {
            $response->addMessage(new DTO\Response\Message($ex->getMessage()));
            $response->setError(new DTO\Response\Error($ex->getMessage(), $ex->getCode()));
        }
    }

    public function removeOrganizationProductElements(DTO\Request\DataSet $request, DTO\Response $response){
        try {
            $row = $request->getDataSet()->getRows()[0];
            $nrorgId = $row['NRORG'];
            $products = $row['PRODUTOS'];
            $result = $this->apiService->removeOrganizationProductElements($nrorgId, $products);
            $response->addDataSet(new DataSource\DataSet('data', $result));
        } catch(\Exception $ex) {
            $response->addMessage(new DTO\Response\Message($ex->getMessage()));
            $response->setError(new DTO\Response\Error($ex->getMessage(), $ex->getCode()));
        }
    }

    public function validateOrganizationByElements(DTO\Request\DataSet $request, DTO\Response $response){
        try {
            $row      = $request->getDataSet()->getRows()[0];
            $products = $row['products'];
            $result   = $this->apiService->validateOrganizationByElements($products);
            $response->addDataSet(new DataSource\DataSet('validation', $result));
        } catch(\Exception $ex) {
            $response->addMessage(new DTO\Response\Message($ex->getMessage()));
            $response->setError(new DTO\Response\Error($ex->getMessage(), $ex->getCode()));
        }
    }
    
    public function synchronizeElementsByProduct(DTO\Request\DataSet $request, DTO\Response $response){
        try {
            $row = $request->getDataSet()->getRows()[0];
            $response->addDataSet(new DataSource\DataSet('synchronize', $this->apiService->synchronizeElementsByProduct($row['productId'], $row['code'])));
        } catch (\Exception $ex){
            $response->setError(new DTO\Response\Error($ex->getMessage(), $ex->getCode()));
        }
    }

    public function synchronizeElementsByIntegrationAndCreate(DTO\Request\DataSet $request, DTO\Response $response){
        try {
            $row = $request->getDataSet()->getRows()[0];
            $response->addDataSet(new DataSource\DataSet('synchronize', $this->apiService->synchronizeElementsByProductAndCreate($row['productId'], $row['code'])));
        } catch (\Exception $ex){
            $response->setError(new DTO\Response\Error($ex->getMessage(), $ex->getCode()));
        }
    }
    
    public function getElementsByDevOpsId(DTO\Request\Filter $request, DTO\Response $response){
        try {
            $filters    = $request->getFilterCriteria()->getConditions();
            $productId  = Utilities::getValueInFilterDataSourceByName($filters, 'PRODUCT_ID', true);
            $cdCliente  = Utilities::getValueInFilterDataSourceByName($filters, 'CDCLIENTE', true);
            $elements   = $this->apiService->getElementsByDevOpsIdWithOrganizationFilter($productId, $cdCliente);
            $response->addDataSet(new DataSource\DataSet('elements', $elements));
        } catch(\Exception $ex) {
            $response->setError(new DTO\Response\Error($ex->getMessage(), $ex->getCode()));
        }
    }

    public function sendCollectionToSM(DTO\Request\DataSet $request, DTO\Response $response){
        try {
            $row = $request->getDataSet()->getRows()[0];
            $response->addDataSet(new DataSource\DataSet('sendCollectionToSM', $this->apiService->sendCollectionToSM($row)));
        } catch (\Exception $ex){
            $response->setError(new DTO\Response\Error($ex->getMessage(), $ex->getCode()));
        }
    }

    public function findOrganizationInformation(Request\Filter $request, Response $response){
        try {
            $conditions  = $request->getFilterCriteria()->getConditions();
            $productId  = Utilities::getValueInFilterDataSourceByName($conditions, 'PRODUCT_ID', true);
            $data       = $this->apiService->findOrganizationInformation($productId, $conditions);
            $response->addDataSet(new DataSource\DataSet('organizationInformation', $data));
        } catch(\Exception $ex) {
            $response->setError(new DTO\Response\Error($ex->getMessage(), $ex->getCode()));
        }
    }

    public function findServerMnemonic(Request\Filter $request, Response $response)
    {
        try {
            $conditions  = $request->getFilterCriteria()->getConditions();
            $productId  = Utilities::getValueInFilterDataSourceByName($conditions, 'PRODUCT_ID', true);
            $serverId  = Utilities::getValueInFilterDataSourceByName($conditions, 'SERVER_ID', true);
            $data = $this->apiService->findServerMnemonic($productId, $serverId, $conditions);
            $response->addDataSet(new DataSource\DataSet('serverMnemonic', $data));
        } catch(\Exception $ex) {
            $response->setError(new DTO\Response\Error($ex->getMessage(), $ex->getCode()));
        }
    }

    public function findProductElement(Request\Filter $request, Response $response)
    {
        try {
            $conditions  = $request->getFilterCriteria()->getConditions();
            $productId  = Utilities::getValueInFilterDataSourceByName($conditions, 'PRODUCT_ID', true);
            $productOrgId  = Utilities::getValueInFilterDataSourceByName($conditions, 'NRPRODUTO_ORG_ID', true);
            $data = $this->apiService->findProductElement($productId, $productOrgId, $conditions);
            $response->addDataSet(new DataSource\DataSet('productElement', $data));
        } catch(\Exception $ex) {
            $response->setError(new DTO\Response\Error($ex->getMessage(), $ex->getCode()));
        }
    }
}