<?php
namespace OrganizationsManagement\Service;

use Doctrine\ORM\EntityManager;
use OrganizationsManagement\Util\InstanceProvider;
use OrganizationsManagement\Util\Messages;
use OrganizationsManagement\Util\Utilities;
use OrganizationsManagement\Util\DBConnection;
use Teknisa\Libs\Util\Environment;
use Teknisa\Model\Geral\Entities\Operador as EntidadeOperador;
use Teknisa\Model\Geral\Entities\Organizacao as EntidadeOrganizacao;
use OrganizationsManagement\Service\Email\EmailSenha;
use OrganizationsManagement\Factory;

class Operador {
    private $entityManager;
    private $factoryOperador;
    private $environment;
    /** @var \Teknisa\Model\Geral\Repositories\Operador */
    private $operadorRepository;

    public function __construct(
        EntityManager $entityManager, Factory\Operador $factoryOperador, DBConnection $dbConnection, Environment $environment
    ){
        $this->dbConnection = $dbConnection;
        $this->entityManager = $entityManager;
        $this->factoryOperador = $factoryOperador;
        $this->environment = $environment;
        $this->operadorRepository = $this->entityManager->getRepository(Utilities::GERAL_ENTITY_DIR . 'Operador');
    }

    public function geraSenhaAleatoria(){
        $tamanhoSenha = 7;
        $caracteres = 'abcdefghijkmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ1234567890#$';
        $quantidadeCaracteres = strlen($caracteres);
        $senhaGerada = '';
        for($i = 1; $i <= $tamanhoSenha; $i++){
            $rand = mt_rand(1, $quantidadeCaracteres);
            $senhaGerada .= $caracteres[$rand-1];
        }
        return $senhaGerada;    
    }

    public function enviaSenhaPorEmail($cdoperador, $nmoperador, $email, $senha){
        $emailSenha = new EmailSenha();
        $emailSenha->enviaEmailSenhaCriada($cdoperador, $nmoperador, $email, $senha);
    }

    public function redefineSenha($idOrganizacao, $emailOperador){
        $defaultEntityManager = $this->entityManager;
        $conexaoDefault = $defaultEntityManager->getConnection();
        $conexao = null;

        try {
            $conexaoDefault->beginTransaction();

            /** @var EntidadeOrganizacao $entidadeOrganizacao */
            $entidadeOrganizacao = InstanceProvider::getRepository()->getOrganizacao()->findOneBy(array('nrorg' => $idOrganizacao));

            $senha        = $this->geraSenhaAleatoria();
            $senhaWeb     = md5($senha);
            $senhaDelphi  = Utilities::geraCriptografiaSenhaDelphi($senha);
            $currentOrgId = $this->environment->getCurrentOrganizationId();

            /** @var EntidadeOperador $defaultEntidadeOperador */
            $defaultEntidadeOperador = $this->operadorRepository->findOneBy(array ('nrorg' => $currentOrgId, 'dsemailoper' => $emailOperador));
            if (!is_object($defaultEntidadeOperador)) {
                throw new \Exception(Messages::ERR_OPERATOR_NOT_FOUND_GENERATE_PASSWORD);
            }
            $defaultEntidadeOperador->setCdsenhaoperweb($senhaWeb);
            $defaultEntidadeOperador->setCdsenhoper($senhaDelphi);
            $defaultEntityManager->persist($defaultEntidadeOperador);

            $codigo = $defaultEntidadeOperador->getCdoperador();
            $nome   = $defaultEntidadeOperador->getNmoperador();

            if($entidadeOrganizacao->isSaas() && !$entidadeOrganizacao->isSecundaria()) {
                $this->entityManager = $this->dbConnection->getOrgEntityManager($idOrganizacao);
                $conexao = $this->entityManager->getConnection();
                $conexao->beginTransaction();
                
                $this->operadorRepository->setEntityManager($this->entityManager);
                /** @var EntidadeOperador $entidadeOperador */
                $entidadeOperador = $this->operadorRepository->findOneBy(array('nrorg' => $idOrganizacao, 'dsemailoper' => $emailOperador));
                if (!is_object($entidadeOperador)) {
                    throw new \Exception(Messages::ERR_OPERATOR_NOT_FOUND_GENERATE_PASSWORD_ORG);
                }
                $entidadeOperador->setCdsenhaoperweb($senhaWeb);
                $entidadeOperador->setCdsenhoper($senhaDelphi);
                $this->entityManager->persist($entidadeOperador);
                
                $this->entityManager->flush();
                $conexao->commit();
            }

            $defaultEntityManager->flush();
            $conexaoDefault->commit();

            $emailSenha = new EmailSenha();
            $emailSenha->enviaEmailSenhaRecriada($codigo, $nome, $emailOperador, $senha);
        } catch(\Exception $ex){
            if($conexao && $conexao->isTransactionActive()) {
                $conexao->rollback();
            }
            if($conexaoDefault->isTransactionActive()) {
                $conexaoDefault->rollback();
            }
            throw $ex;
        }
    }
} 