/* Tabela IMPOSERIE: */
/* Descrio: A trigger insere registros na tabela LOGOPERA a partir de alteraes na
tabela IMPOSERIE */
/* Objetivo: Gerar um log que vai registrar as alteraes na tabela IMPOSERIE, utilizando
a tabela Geral de Logs de Operaes */

IF EXISTS (SELECT * FROM SYSOBJECTS O, SYSUSERS U
           WHERE O.NAME = 'LOG_FIS_IMPOSERIE' AND
                 TYPE   = 'TR'                AND
                 U.UID  = O.UID               AND
                 U.NAME = USER)
DROP TRIGGER LOG_FIS_IMPOSERIE
GO

CREATE TRIGGER LOG_FIS_IMPOSERIE
ON IMPOSERIE
FOR DELETE, INSERT, UPDATE
AS

   DECLARE @V_USUARIO        VARCHAR(40);
   DECLARE @V_TIPO           VARCHAR(1);
   DECLARE @V_ITPESQ         VARCHAR(40);
   DECLARE @V_DESC           VARCHAR(300);
   DECLARE @V_DTOPER         DATETIME;

/*VARIVEIS DO CURSOR C_CURSOR*/

   DECLARE @V_ID_COMANDO      VARCHAR(1);
   DECLARE @V_OLD_CDEMPRESA   VARCHAR(2);
   DECLARE @V_OLD_CDINSCESTA  VARCHAR(16);
   DECLARE @V_OLD_CDINSCMUNI  VARCHAR(16);
   DECLARE @V_OLD_CDSERIE     VARCHAR(5);
   DECLARE @V_OLD_CDIMPOSTO   VARCHAR(2);

   DECLARE @V_NEW_CDEMPRESA   VARCHAR(2);
   DECLARE @V_NEW_CDINSCESTA  VARCHAR(16);
   DECLARE @V_NEW_CDINSCMUNI  VARCHAR(16);
   DECLARE @V_NEW_CDSERIE     VARCHAR(5);
   DECLARE @V_NEW_CDIMPOSTO   VARCHAR(2);

BEGIN
   SET @V_TIPO = '0';
   SET @V_DESC = '';

   EXEC NMUSUARIO @V_USUARIO;
   SET @V_USUARIO = ISNULL(@V_USUARIO, '');

   DECLARE C_CURSOR CURSOR FOR
      SELECT
         'D' IDCOMANDO,
         CDEMPRESA,
         CDINSCESTA,
         CDINSCMUNI,
         CDSERIE,
         CDIMPOSTO,
         NULL,
         NULL,
         NULL,
         NULL,
         NULL
      FROM
         DELETED
      WHERE
         CDEMPRESA+CDINSCESTA+CDINSCMUNI+CDSERIE+CDIMPOSTO NOT IN (SELECT CDEMPRESA+CDINSCESTA+CDINSCMUNI+CDSERIE+CDIMPOSTO FROM INSERTED)

      UNION ALL

      SELECT
         'U' IDCOMANDO,
         DELETED.CDEMPRESA,
         DELETED.CDINSCESTA,
         DELETED.CDINSCMUNI,
         DELETED.CDSERIE,
         DELETED.CDIMPOSTO,
         INSERTED.CDEMPRESA,
         INSERTED.CDINSCESTA,
         INSERTED.CDINSCMUNI,
         INSERTED.CDSERIE,
         INSERTED.CDIMPOSTO
      FROM
         INSERTED,
         DELETED
      WHERE
         INSERTED.CDEMPRESA  = DELETED.CDEMPRESA    AND
         INSERTED.CDINSCESTA = DELETED.CDINSCESTA   AND
         INSERTED.CDINSCMUNI = DELETED.CDINSCMUNI   AND
         INSERTED.CDSERIE    = DELETED.CDSERIE      AND
         INSERTED.CDIMPOSTO  = DELETED.CDIMPOSTO

      UNION ALL

      SELECT
         'I' IDCOMANDO,
         NULL,
         NULL,
         NULL,
         NULL,
         NULL,
         CDEMPRESA,
         CDINSCESTA,
         CDINSCMUNI,
         CDSERIE,
         CDIMPOSTO
      FROM
         INSERTED
      WHERE
         CDEMPRESA+CDINSCESTA+CDINSCMUNI+CDSERIE+CDIMPOSTO NOT IN (SELECT CDEMPRESA+CDINSCESTA+CDINSCMUNI+CDSERIE+CDIMPOSTO FROM DELETED);

      OPEN C_CURSOR;
      FETCH NEXT FROM C_CURSOR INTO @V_ID_COMANDO,
                                    @V_OLD_CDEMPRESA,
                                    @V_OLD_CDINSCESTA,
                                    @V_OLD_CDINSCMUNI,
                                    @V_OLD_CDSERIE,
                                    @V_OLD_CDIMPOSTO,

                                    @V_NEW_CDEMPRESA,
                                    @V_NEW_CDINSCESTA,
                                    @V_NEW_CDINSCMUNI,
                                    @V_NEW_CDSERIE,
                                    @V_NEW_CDIMPOSTO;

   WHILE (@@FETCH_STATUS = 0) BEGIN
      SET @V_DESC = '';
      IF @V_ID_COMANDO = 'D' BEGIN
         IF @V_OLD_CDIMPOSTO IS NOT NULL BEGIN
            SET @V_DESC = @V_DESC + 'Cod=' + @V_OLD_CDIMPOSTO;
         END;

         SET @V_TIPO     = '3';
         SET @V_ITPESQ   = @V_OLD_CDEMPRESA + @V_OLD_CDINSCESTA + @V_OLD_CDINSCMUNI + @V_OLD_CDSERIE + @V_OLD_CDIMPOSTO;
      END
      ELSE IF @V_ID_COMANDO= 'U' /*UPDATING*/ BEGIN
         IF (@V_OLD_CDIMPOSTO <> @V_NEW_CDIMPOSTO) BEGIN
            SET @V_TIPO    = '2';
            SET @V_ITPESQ  = @V_OLD_CDEMPRESA + @V_OLD_CDINSCESTA + @V_OLD_CDINSCMUNI + @V_OLD_CDSERIE + @V_OLD_CDIMPOSTO;
         END;
      END
      ELSE BEGIN
         SET @V_TIPO     = '1';
         SET @V_ITPESQ   = @V_NEW_CDEMPRESA + @V_NEW_CDINSCESTA + @V_NEW_CDINSCMUNI + @V_NEW_CDSERIE + @V_NEW_CDIMPOSTO;
      END;

      IF @V_NEW_CDIMPOSTO IS NOT NULL BEGIN
         SET @V_DESC = @V_DESC + 'Cod=' + @V_NEW_CDIMPOSTO;
      END;

      IF @V_TIPO <> '0' BEGIN
         EXEC NMUSUARIO @V_USUARIO;
         IF NOT @V_DESC IS NULL BEGIN
            IF NOT @V_USUARIO IS NULL BEGIN
               SET @V_DTOPER = GETDATE();
               INSERT INTO LOGOPERA
                  (NMTABELA,   DTOPERACAO, NMRESPOPER,
                   IDTIPOOPER, CDPESQITEM, DSOPERACAO)
               VALUES
                  ('IMPOSERIE',  @V_DTOPER, @V_USUARIO,
                   @V_TIPO,      @V_ITPESQ, @V_DESC);
            END;
         END;
      END;
      FETCH NEXT FROM C_CURSOR INTO @V_ID_COMANDO,
                                    @V_OLD_CDEMPRESA,
                                    @V_OLD_CDINSCESTA,
                                    @V_OLD_CDINSCMUNI,
                                    @V_OLD_CDSERIE,
                                    @V_OLD_CDIMPOSTO,

                                    @V_NEW_CDEMPRESA,
                                    @V_NEW_CDINSCESTA,
                                    @V_NEW_CDINSCMUNI,
                                    @V_NEW_CDSERIE,
                                    @V_NEW_CDIMPOSTO;
   END;
   CLOSE      C_CURSOR;
   DEALLOCATE C_CURSOR;
END;
GO

-- VERSAO --
EXEC ATUVEROBJ 'LOG_FIS_IMPOSERIE','4.11.105';
GO
